from cms.cache import _get_cache_key
from cms.models import Page, get_cms_setting
from cms.utils.moderator import use_draft
from django import template
from django.conf import settings
from django.core.cache import cache
from django.core.exceptions import ObjectDoesNotExist
from django.utils.translation import get_language


register = template.Library()

NO_EXTENSION = 'no_extension'
CMS_HELPERS_PAGE_TITLEEXTENSION_CACHE = 'CMS_HELPERS_PAGE_TITLEEXTENSION_CACHE'


def is_caching_desired(do_cache):
    if do_cache is not None:
        return do_cache
    custom_setting = getattr(settings, CMS_HELPERS_PAGE_TITLEEXTENSION_CACHE, None)
    if custom_setting is not None:
        return custom_setting
    return get_cms_setting('PLUGIN_CACHE')


@register.simple_tag(takes_context=True)
def page_titleextension(context, page_id, extension, do_cache=None):
    do_cache = is_caching_desired(do_cache=do_cache)
    request = context.get('request')
    is_draft = request and hasattr(request, 'user') and use_draft(request)

    if do_cache:
        lang = get_language()
        site_id = settings.SITE_ID
        cache_duration = get_cms_setting('CACHE_DURATIONS')['content']
        page_cache_key = _get_cache_key('page_titleextension', page_id, lang, site_id)
        cache_key = '{}_{}_{}'.format(page_cache_key, is_draft, extension)

        cached = cache.get(cache_key)
        if cached:
            if cached == NO_EXTENSION:
                return None
            return cached

    try:
        page = Page.objects.get(pk=page_id)
        if is_draft:
            page = page.get_draft_object()
        else:
            page = page.get_public_object()
    except NameError:
        raise ImportError(
            'django-cms is required when using page_titleextension tag')
    except Page.DoesNotExist:
        return None

    if not page:
        return None

    try:
        title_extension = getattr(page.get_title_obj(), extension)
    except ObjectDoesNotExist:
        if do_cache:
            cache.set(cache_key, NO_EXTENSION, timeout=cache_duration)
        return None
    else:
        if do_cache:
            cache.set(cache_key, title_extension, timeout=cache_duration)
        return title_extension



from cms.extensions.toolbar import ExtensionToolbar
from cms.utils import get_language_list
from django.utils.encoding import force_text
from django.utils.translation import get_language_info


class TitleExtensionToolbar(ExtensionToolbar):
    model = None
    insert_after = None

    def get_item_position(self, menu):
        position = None
        for items in menu._memo.values():
            for item in items:
                if force_text(getattr(item, 'name', None)) in (
                    force_text(self.insert_after),
                    '{0}...'.format(self.insert_after)
                ):
                    position = menu._item_position(item) + 1
                    break

        return position

    def populate(self):
        current_page_menu = self._setup_extension_toolbar()
        if not current_page_menu or not self.page:
            return

        languages = get_language_list(self.current_site.pk)
        is_single_lang = len(languages) < 2
        position = self.get_item_position(current_page_menu)
        urls = self.get_title_extension_admin()
        page = self._get_page()
        titleset = page.title_set.filter(language__in=languages)

        if hasattr(self.toolbar, 'edit_mode_active'):
            not_edit_mode = not self.toolbar.edit_mode_active
        else:
            not_edit_mode = not self.toolbar.edit_mode

        extended_menu = current_page_menu if is_single_lang else (
            current_page_menu.get_or_create_menu(
                key='{0}_menu'.format(self.model._meta.db_table),
                verbose_name=self.model._meta.verbose_name,
                position=position, disabled=not_edit_mode))

        nodes = [(title_extension, url, title) for (
            (title_extension, url), title) in zip(urls, titleset)]

        for title_extension, url, title in nodes:
            item_position = position if is_single_lang else None
            language_str = get_language_info(title.language)['name_translated']
            name = '{0}{1}'.format(
                '' if is_single_lang else (language_str + ' '),
                self.model._meta.verbose_name)
            extended_menu.add_modal_item(
                name, url=url, disabled=not_edit_mode, position=item_position)



from urllib import parse as urlparse

from anylink.extensions import BaseLink
from cms.models.fields import PageField
from cms.utils.conf import get_cms_setting
from django.conf import settings
from django.core.cache import cache
from django.db import models
from django.utils.translation import get_language
from django.utils.translation import ugettext_lazy as _


class CmsPageLink(BaseLink):
    name = 'page'
    verbose_name = _('CMS Page')
    provided_fields = ('page', 'anchor')

    def configure_model(self, model):
        model.add_to_class(
            self.get_name(), PageField(blank=True, null=True))
        model.add_to_class(
            'anchor', models.CharField(max_length=256, null=True, blank=True))

    def get_absolute_url(self, link):
        cache_key = '{0}anylink-page-url:{1}:{2}'.format(
            get_cms_setting('CACHE_PREFIX'), str(link.page_id), get_language())

        url = cache.get(cache_key)
        if url is None:
            if hasattr(link.page, 'site_id') and settings.SITE_ID != link.page.site_id:
                url = '//{domain}{url}'.format(
                    domain=link.page.site.domain,
                    url=link.page.get_absolute_url()
                )
            elif hasattr(link.page, 'node') and settings.SITE_ID != link.page.node.site_id:
                url = '//{domain}{url}'.format(
                    domain=link.page.node.site.domain,
                    url=link.page.get_absolute_url()
                )
            else:
                url = link.page.get_absolute_url()

            cache.set(cache_key, url, get_cms_setting('CACHE_DURATIONS')['content'])

        if link.anchor:
            parsed = urlparse.urlparse(url)
            parsed = parsed._replace(fragment=link.anchor)
            url = parsed.geturl()

        return url



import os

from django import forms
from django.db import models
from django.utils.translation import ugettext
from filer.fields.file import AdminFileFormField as BaseAdminFileFormField
from filer.fields.file import FilerFileField as BaseFilerFileField


class AdminFileFormField(BaseAdminFileFormField):
    def __init__(self, *args, **kwargs):
        self.extensions = kwargs.pop('extensions', None)
        self.alt_text_required = kwargs.pop('alt_text_required', True)
        super().__init__(*args, **kwargs)

    def clean(self, value):
        cleaned = super(AdminFileFormField, self).clean(value)

        if not cleaned:
            return cleaned

        if self.extensions:
            extension = os.path.splitext(cleaned.file.name)[1].strip('.').lower()
            if extension not in self.extensions:
                raise forms.ValidationError(
                    ugettext('Invalid file extension, allowed extensions: {0}'.format(
                        ', '.join(self.extensions))))

        if self.alt_text_required:
            if hasattr(cleaned, 'default_alt_text') and not cleaned.default_alt_text:
                raise forms.ValidationError(
                    ugettext('Alternative text is missing for this file.'))

        return cleaned


class FilerFileField(BaseFilerFileField):
    default_form_class = AdminFileFormField

    def __init__(self, verbose_name=None, *args, **kwargs):
        kwargs['verbose_name'] = verbose_name
        if 'related_name' not in kwargs:
            kwargs['related_name'] = '+'
        if kwargs.pop('blank', False) or kwargs.pop('null', False):
            kwargs['null'] = True
            kwargs['blank'] = True
        kwargs.setdefault('on_delete', models.CASCADE)

        self.extensions = kwargs.pop('extensions', None)
        self.alt_text_required = kwargs.pop('alt_text_required', True)

        super().__init__(*args, **kwargs)

    def formfield(self, **kwargs):
        defaults = {
            'extensions': self.extensions,
            'alt_text_required': self.alt_text_required,
        }
        defaults.update(kwargs)
        return super().formfield(**defaults)



import os
from codecs import open

from setuptools import setup, find_packages


BASE_DIR = os.path.abspath(os.path.dirname(__file__))
VERSION = __import__('cms_helpers').__version__


with open(os.path.join(BASE_DIR, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()


setup(
    name='django-cms-helpers',
    version=VERSION,
    description='django-cms-helpers is a collection of helpers when working with django-cms.',
    long_description=long_description,
    url='https://github.com/moccu/django-cms-helpers',
    project_urls={
        'Bug Reports': 'https://github.com/moccu/django-cms-helpers/issues',
        'Source': 'https://github.com/moccu/django-cms-helpers',
    },
    author='moccu',
    author_email='info@moccu.com',
    packages=find_packages(exclude=['tests', 'tests.*']),
    install_requires=['django-cms'],
    include_package_data=True,
    keywords='django',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
    ],
)
